@@ -207,7 +207,6 @@ class _Device(ABC, HasListeners):
 class Keyboard(_Device):
     def __init__(self, core: Core, wlr_device: InputDevice, keyboard: WlrKeyboard):
         super().__init__(core, wlr_device)
-        self.qtile = core.qtile
         self.seat = core.seat
         self.keyboard = keyboard
         self.grabbed_keys = core.grabbed_keys
@@ -223,7 +222,7 @@ class Keyboard(_Device):
     def finalize(self) -> None:
         super().finalize()
         self.core.keyboards.remove(self)
-        if self.core.seat.keyboard == self.keyboard:
+        if self.core.seat.get_keyboard() == self.keyboard:
             # If this is the active keyboard and we have other keyboards enabled, set
             # the previous keyboard as the new active keyboard.
             if self.core.keyboards:
@@ -247,12 +246,7 @@ class Keyboard(_Device):
         self.seat.keyboard_notify_modifiers(self.keyboard.modifiers)
 
     def _on_key(self, _listener: Listener, event: KeyboardKeyEvent) -> None:
-        if self.qtile is None:
-            # shushes mypy
-            self.qtile = self.core.qtile
-            assert self.qtile is not None
-
-        self.core.idle.notify_activity(self.seat)
+        self.qtile = self.core.qtile
 
         if event.state == KEY_PRESSED and not self.core.exclusive_client:
             # translate libinput keycode -> xkbcommon
