@@ -21,6 +21,7 @@
 from __future__ import annotations
 
 import asyncio
+import faulthandler
 import io
 import logging
 import os
@@ -44,7 +45,7 @@ from libqtile.command.client import InteractiveCommandClient
 from libqtile.command.interface import IPCCommandServer, QtileCommandInterface
 from libqtile.config import Click, Drag, Key, KeyChord, Match, Mouse, Rule
 from libqtile.config import ScratchPad as ScratchPadConfig
-from libqtile.config import Screen
+from libqtile.config import Screen, ScreenRect
 from libqtile.core.lifecycle import lifecycle
 from libqtile.core.loop import LoopContext, QtileEventLoopPolicy
 from libqtile.core.state import QtileState
@@ -81,7 +82,7 @@ class Qtile(CommandObject):
         state: str | None = None,
         socket_path: str | None = None,
     ) -> None:
-        self.core = kore
+        self.core: base.Core = kore
         self.config = config
         self.no_spawn = no_spawn
         self._state: QtileState | str | None = state
@@ -210,7 +211,13 @@ class Qtile(CommandObject):
         # Set the event loop policy to facilitate access to main event loop
         asyncio.set_event_loop_policy(QtileEventLoopPolicy(self))
         self._stopped_event = asyncio.Event()
-        self.core.setup_listener(self)
+        self.core.qtile = self
+        self.load_config(initial=True)
+        self.core.setup_listener()
+
+        faulthandler.enable(all_threads=True)
+        faulthandler.register(signal.SIGUSR2, all_threads=True)
+
         try:
             async with LoopContext(
                 {
@@ -218,13 +225,11 @@ class Qtile(CommandObject):
                     signal.SIGINT: self.stop,
                     signal.SIGHUP: self.stop,
                     signal.SIGUSR1: self.reload_config,
-                    signal.SIGUSR2: self.restart,
                 }
             ), ipc.Server(
                 self._prepare_socket_path(self.socket_path),
                 self.server.call,
             ):
-                self.load_config(initial=True)
                 await self._stopped_event.wait()
         finally:
             self.finalize()
@@ -352,20 +357,22 @@ class Qtile(CommandObject):
         screens = []
 
         if hasattr(self.config, "fake_screens"):
-            screen_info = [(s.x, s.y, s.width, s.height) for s in self.config.fake_screens]
+            screen_info = [
+                ScreenRect(s.x, s.y, s.width, s.height) for s in self.config.fake_screens
+            ]
             config = self.config.fake_screens
         else:
             # Alias screens with the same x and y coordinates, taking largest
             xywh = {}  # type: dict[tuple[int, int], tuple[int, int]]
-            for sx, sy, sw, sh in self.core.get_screen_info():
-                pos = (sx, sy)
+            for info in self.core.get_screen_info():
+                pos = (info.x, info.y)
                 width, height = xywh.get(pos, (0, 0))
-                xywh[pos] = (max(width, sw), max(height, sh))
+                xywh[pos] = (max(width, info.width), max(height, info.height))
 
-            screen_info = [(x, y, w, h) for (x, y), (w, h) in xywh.items()]
+            screen_info = [ScreenRect(x, y, w, h) for (x, y), (w, h) in xywh.items()]
             config = self.config.screens
 
-        for i, (x, y, w, h) in enumerate(screen_info):
+        for i, info in enumerate(screen_info):
             if i + 1 > len(config):
                 scr = Screen()
             else:
@@ -387,9 +394,9 @@ class Qtile(CommandObject):
 
             # If the screen has changed position and/or size, or is a new screen then make sure that any gaps/bars
             # are reconfigured
-            reconfigure_gaps = ((x, y, w, h) != (scr.x, scr.y, scr.width, scr.height)) or (
-                i + 1 > len(self.screens)
-            )
+            reconfigure_gaps = (
+                (info.x, info.y, info.width, info.height) != (scr.x, scr.y, scr.width, scr.height)
+            ) or (i + 1 > len(self.screens))
 
             if not hasattr(scr, "group"):
                 # Ensure that this screen actually *has* a group, as it won't get
@@ -400,7 +407,16 @@ class Qtile(CommandObject):
                 # a group anyway.
                 scr.group = grp
 
-            scr._configure(self, i, x, y, w, h, grp, reconfigure_gaps=reconfigure_gaps)
+            scr._configure(
+                self,
+                i,
+                info.x,
+                info.y,
+                info.width,
+                info.height,
+                grp,
+                reconfigure_gaps=reconfigure_gaps,
+            )
             screens.append(scr)
 
         for screen in self.screens:
@@ -448,7 +464,7 @@ class Qtile(CommandObject):
             for cmd in key.commands:
                 if cmd.check(self):
                     status, val = self.server.call(
-                        (cmd.selectors, cmd.name, cmd.args, cmd.kwargs)
+                        (cmd.selectors, cmd.name, cmd.args, cmd.kwargs, False)
                     )
                     if status in (interface.ERROR, interface.EXCEPTION):
                         logger.error("KB command error %s: %s", cmd.name, val)
@@ -796,7 +812,9 @@ class Qtile(CommandObject):
             if isinstance(m, Click):
                 for i in m.commands:
                     if i.check(self):
-                        status, val = self.server.call((i.selectors, i.name, i.args, i.kwargs))
+                        status, val = self.server.call(
+                            (i.selectors, i.name, i.args, i.kwargs, False)
+                        )
                         if status in (interface.ERROR, interface.EXCEPTION):
                             logger.error("Mouse command error %s: %s", i.name, val)
                         handled = True
@@ -805,7 +823,7 @@ class Qtile(CommandObject):
             ):
                 if m.start:
                     i = m.start
-                    status, val = self.server.call((i.selectors, i.name, i.args, i.kwargs))
+                    status, val = self.server.call((i.selectors, i.name, i.args, i.kwargs, False))
                     if status in (interface.ERROR, interface.EXCEPTION):
                         logger.error("Mouse command error %s: %s", i.name, val)
                         continue
@@ -844,7 +862,7 @@ class Qtile(CommandObject):
             for i in cmd:
                 if i.check(self):
                     status, val = self.server.call(
-                        (i.selectors, i.name, i.args + (rx + dx, ry + dy), i.kwargs)
+                        (i.selectors, i.name, i.args + (rx + dx, ry + dy), i.kwargs, False)
                     )
                     if status in (interface.ERROR, interface.EXCEPTION):
                         logger.error("Mouse command error %s: %s", i.name, val)
@@ -1118,7 +1136,7 @@ class Qtile(CommandObject):
         return list(self.widgets_map.keys())
 
     @expose_command()
-    def to_layout_index(self, index: str, name: str | None = None) -> None:
+    def to_layout_index(self, index: int, name: str | None = None) -> None:
         """
         Switch to the layout with the given index in self.layouts.
 
@@ -1224,7 +1242,7 @@ class Qtile(CommandObject):
 
     @expose_command()
     def spawn(
-        self, cmd: str | list[str], shell: bool = False, env: dict[str, str] = dict()
+        self, cmd: list[str] | str, shell: bool = False, env: dict[str, str] = dict()
     ) -> int:
         """
         Spawn a new process.
