@@ -29,8 +29,8 @@ import collections
 
 import libqtile.hook
 from libqtile.backend.base import Static
-from libqtile.command import lazy
 from libqtile.config import Group, Key, Match, Rule
+from libqtile.lazy import lazy
 from libqtile.log_utils import logger
 
 
@@ -254,4 +254,5 @@ class DGroups:
             del self.timeout[client]
 
         logger.debug("Deleting %s in %ss", group, self.delay)
-        self.timeout[client] = self.qtile.call_later(self.delay, delete_client)
+        if client not in self.timeout:
+            self.timeout[client] = self.qtile.call_later(self.delay, delete_client)
