@@ -32,7 +32,7 @@ from typing import TYPE_CHECKING
 
 from libqtile import hook, utils
 from libqtile.backend.base import FloatStates
-from libqtile.command.base import CommandObject
+from libqtile.command.base import CommandObject, expose_command
 from libqtile.log_utils import logger
 
 if TYPE_CHECKING:
@@ -48,7 +48,8 @@ class _Group(CommandObject):
     A group is identified by its name but displayed in GroupBox widget by its label.
     """
 
-    def __init__(self, name, layout=None, label=None):
+    def __init__(self, name, layout=None, label=None, screen_affinity=None):
+        self.screen_affinity = screen_affinity
         self.name = name
         self.label = name if label is None else label
         self.custom_layout = layout  # will be set on _configure
@@ -127,7 +128,7 @@ class _Group(CommandObject):
                 return
         logger.error("No such layout: %s", layout)
 
-    def use_layout(self, index):
+    def use_layout(self, index: int):
         assert -len(self.layouts) <= index < len(self.layouts), "layout index out of bounds"
         self.layout.hide()
         self.current_layout = index % len(self.layouts)
@@ -231,7 +232,9 @@ class _Group(CommandObject):
             hook.fire("focus_change")
             self.layout_all(warp)
 
+    @expose_command()
     def info(self):
+        """Returns a dictionary of info for this group"""
         return dict(
             name=self.name,
             label=self.label,
@@ -254,12 +257,14 @@ class _Group(CommandObject):
             win._float_state = FloatStates.FULLSCREEN
         elif self.floating_layout.match(win) and not win.fullscreen:
             win._float_state = FloatStates.FLOATING
+            if self.qtile.config.floats_kept_above:
+                win.keep_above(enable=True)
         if win.floating and not win.fullscreen:
-            self.floating_layout.add(win)
+            self.floating_layout.add_client(win)
         if not win.floating or win.fullscreen:
             self.tiled_windows.add(win)
             for i in self.layouts:
-                i.add(win)
+                i.add_client(win)
         if focus:
             self.focus(win, warp=True, force=force)
 
@@ -318,7 +323,7 @@ class _Group(CommandObject):
                         i.remove(win)
                         if win is self.current_window:
                             i.blur()
-                    self.floating_layout.add(win)
+                    self.floating_layout.add_client(win)
                     if win is self.current_window:
                         self.floating_layout.focus(win)
         else:
@@ -327,7 +332,7 @@ class _Group(CommandObject):
             # A window that was fullscreen should only be added if it was not a tiled window
             if win not in self.tiled_windows:
                 for i in self.layouts:
-                    i.add(win)
+                    i.add_client(win)
                 self.tiled_windows.add(win)
             if win is self.current_window:
                 for i in self.layouts:
@@ -358,14 +363,12 @@ class _Group(CommandObject):
                     return i
         raise RuntimeError("Invalid selection: {}".format(name))
 
-    def cmd_setlayout(self, layout):
+    @expose_command()
+    def setlayout(self, layout):
         self.layout = layout
 
-    def cmd_info(self):
-        """Returns a dictionary of info for this group"""
-        return self.info()
-
-    def cmd_toscreen(self, screen=None, toggle=False):
+    @expose_command()
+    def toscreen(self, screen=None, toggle=False):
         """Pull a group to a specified screen.
 
         Parameters
@@ -431,13 +434,15 @@ class _Group(CommandObject):
     def get_next_group(self, skip_empty=False, skip_managed=False):
         return self._get_group(1, skip_empty, skip_managed)
 
-    def cmd_unminimize_all(self):
+    @expose_command()
+    def unminimize_all(self):
         """Unminimise all windows in this group"""
         for win in self.windows:
             win.minimized = False
         self.layout_all()
 
-    def cmd_next_window(self):
+    @expose_command()
+    def next_window(self):
         """
         Focus the next window in group.
 
@@ -461,7 +466,8 @@ class _Group(CommandObject):
             )
         self.focus(nxt, True)
 
-    def cmd_prev_window(self):
+    @expose_command()
+    def prev_window(self):
         """
         Focus the previous window in group.
 
@@ -485,7 +491,8 @@ class _Group(CommandObject):
             )
         self.focus(nxt, True)
 
-    def cmd_focus_back(self):
+    @expose_command()
+    def focus_back(self):
         """
         Focus the window that had focus before the current one got it.
 
@@ -500,7 +507,8 @@ class _Group(CommandObject):
         else:
             self.focus(win)
 
-    def cmd_focus_by_name(self, name):
+    @expose_command()
+    def focus_by_name(self, name):
         """
         Focus the first window with the given name. Do nothing if the name is
         not found.
@@ -510,7 +518,8 @@ class _Group(CommandObject):
                 self.focus(win)
                 break
 
-    def cmd_info_by_name(self, name):
+    @expose_command()
+    def info_by_name(self, name):
         """
         Get the info for the first window with the given name without giving it
         focus. Do nothing if the name is not found.
@@ -519,11 +528,40 @@ class _Group(CommandObject):
             if win.name == name:
                 return win.info()
 
-    def cmd_switch_groups(self, name):
-        """Switch position of current group with name"""
-        self.qtile.cmd_switch_groups(self.name, name)
+    @expose_command()
+    def focus_by_index(self, index: int) -> None:
+        """
+        Change to the window at the specified index in the current group.
+        """
+        windows = self.windows
+        if index < 0 or index > len(windows) - 1:
+            return
 
-    def cmd_set_label(self, label):
+        self.focus(windows[index])
+
+    @expose_command()
+    def swap_window_order(self, new_location: int) -> None:
+        """
+        Change the order of the current window within the current group.
+        """
+        if new_location < 0 or new_location > len(self.windows) - 1:
+            return
+
+        windows = self.windows
+        current_window_index = windows.index(self.current_window)
+
+        windows[current_window_index], windows[new_location] = (
+            windows[new_location],
+            windows[current_window_index],
+        )
+
+    @expose_command()
+    def switch_groups(self, name):
+        """Switch position of current group with name"""
+        self.qtile.switch_groups(self.name, name)
+
+    @expose_command()
+    def set_label(self, label):
         """
         Set the display name of current group to be used in GroupBox widget.
         If label is None, the name of the group is used as display name.
